﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Extensions.Caching.Redis;
using Microsoft.Extensions.DependencyInjection;

namespace Falcon.Extend
{
    /// <summary>
    /// ServiceCollection扩展
    /// </summary>
    public static class ServiceCollectionExtend
    {
        /// <summary>
        /// 通过提供RedisCacheOptions配置Redis实例
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="option">RedisCache参数</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddRedis(this IServiceCollection services,RedisCacheOptions option) {
            services.AddDistributedRedisCache(op => {
                op.InstanceName = option.InstanceName;
                op.Configuration = option.Configuration;
            });
            services.AddTransient<ICacheProvider,RedisCacheProvider>();
            return services;
        }

        /// <summary>
        /// 添加微软Json序列化器
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddMsJsonProvider(this IServiceCollection services) {
            return services.AddSingleton<IJsonProvider,MsJsonProvider>();
        }
    }
}
